@@ -24,6 +24,9 @@
 
 #include <gtk/gtk.h>
 
+#include <gdl/gdl-dock.h>
+
+
 G_BEGIN_DECLS
 
 /* standard macros */
@@ -39,6 +42,16 @@ typedef struct _GdlDockBar        GdlDockBar;
 typedef struct _GdlDockBarClass   GdlDockBarClass;
 typedef struct _GdlDockBarPrivate GdlDockBarPrivate;
 
+/**
+ * GdlDockBarStyle:
+ * @GDL_DOCK_BAR_ICONS: Buttons display only icons in the dockbar.
+ * @GDL_DOCK_BAR_TEXT: Buttons display only text labels in the dockbar.
+ * @GDL_DOCK_BAR_BOTH: Buttons display text and icons in the dockbar.
+ * @GDL_DOCK_BAR_AUTO: identical to @GDL_DOCK_BAR_BOTH.
+ *
+ * Used to customize the appearance of a #GdlDockBar.
+ *
+ **/
 typedef enum {
     GDL_DOCK_BAR_ICONS,
     GDL_DOCK_BAR_TEXT,
@@ -49,6 +62,7 @@ typedef enum {
 struct _GdlDockBar {
     GtkBox parent;
 
+    /*< private >*/
     GdlDock *dock;
 
     GdlDockBarPrivate *priv;
@@ -58,17 +72,20 @@ struct _GdlDockBarClass {
     GtkBoxClass parent_class;
 };
 
-GType      gdl_dock_bar_get_type            (void); 
+GType      gdl_dock_bar_get_type            (void);
 
 GtkWidget *gdl_dock_bar_new                 (GdlDock     *dock);
 
-GtkOrientation gdl_dock_bar_get_orientation (GdlDockBar *dockbar);
-void           gdl_dock_bar_set_orientation (GdlDockBar *dockbar,
-                                             GtkOrientation orientation);
 void           gdl_dock_bar_set_style       (GdlDockBar *dockbar,
                                              GdlDockBarStyle style);
 GdlDockBarStyle gdl_dock_bar_get_style      (GdlDockBar *dockbar);
 
+#ifndef GDL_DISABLE_DEPRECATED
+GtkOrientation gdl_dock_bar_get_orientation (GdlDockBar *dockbar) G_GNUC_DEPRECATED_FOR(gtk_orientable_get_orientation);
+void           gdl_dock_bar_set_orientation (GdlDockBar *dockbar,
+                                             GtkOrientation orientation) G_GNUC_DEPRECATED_FOR(gtk_orientable_set_orientation);
+#endif
+
 G_END_DECLS
 
 #endif /* __GDL_DOCK_BAR_H__ */
