@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
  * gdl-dock-master.c - Object which manages a dock ring
  *
@@ -36,6 +36,29 @@
 #include "libgdlmarshal.h"
 #include "libgdltypebuiltins.h"
 
+/**
+ * SECTION:gdl-dock-master
+ * @title: GdlDockMaster
+ * @short_description: Manage all dock widgets
+ * @stability: Unstable
+ * @see_also: #GdlDockObject, #GdlDockNotebook, #GdlDockPaned
+ *
+ * For the toplevel docks to be able to interact with each other, when the user
+ * drags items from one place to another, they're all kept in a user-invisible
+ * and automatic object called the master. To participate in docking operations
+ * every #GdlDockObject must have the same master, the binding to the master is
+ * done automatically.  The master also keeps track of the manual items,
+ * mostly those created with gdl_dock_*_new functions which are in the dock.
+ * This is so the user doesn't need to keep track of them, but can perform
+ * operations like hiding and such.
+ *
+ * The master is responsible for creating automatically compound widgets.
+ * When the user drops a widget on a simple one, a notebook or a paned compound
+ * widget containing both widgets is created and replace it.
+ * Such widgets are destroyed automatically too when they have less than two
+ * children.
+ */
+
 /* ----- Private prototypes ----- */
 
 static void     gdl_dock_master_class_init    (GdlDockMasterClass *klass);
@@ -54,13 +77,14 @@ static void     gdl_dock_master_get_property  (GObject            *object,
 static void     _gdl_dock_master_remove       (GdlDockObject      *object,
                                                GdlDockMaster      *master);
 
-static void     gdl_dock_master_drag_begin    (GdlDockItem        *item, 
+static void     gdl_dock_master_drag_begin    (GdlDockItem        *item,
                                                gpointer            data);
 static void     gdl_dock_master_drag_end      (GdlDockItem        *item,
                                                gboolean            cancelled,
                                                gpointer            data);
-static void     gdl_dock_master_drag_motion   (GdlDockItem        *item, 
-                                               gint                x, 
+static void     gdl_dock_master_drag_motion   (GdlDockItem        *item,
+                                               GdkDevice          *device,
+                                               gint                x,
                                                gint                y,
                                                gpointer            data);
 
@@ -68,7 +92,9 @@ static void     _gdl_dock_master_foreach      (gpointer            key,
                                                gpointer            value,
                                                gpointer            user_data);
 
-static void     gdl_dock_master_xor_rect      (GdlDockMaster      *master);
+static void     gdl_dock_master_show_preview   (GdlDockMaster      *master);
+
+static void     gdl_dock_master_hide_preview   (GdlDockMaster      *master);
 
 static void     gdl_dock_master_layout_changed (GdlDockMaster     *master);
 
@@ -92,10 +118,9 @@ enum {
 struct _GdlDockMasterPrivate {
     gint            number;             /* for naming nameless manual objects */
     gchar          *default_title;
-    
-    gboolean        rect_drawn;
+
     GdlDock        *rect_owner;
-    
+
     GdlDockRequest *drag_request;
 
     /* source id for the idle handler to emit a layout_changed signal */
@@ -108,7 +133,7 @@ struct _GdlDockMasterPrivate {
      */
     GHashTable     *locked_items;
     GHashTable     *unlocked_items;
-    
+
     GdlSwitcherStyle switcher_style;
 
     /* Window for preview rect */
@@ -146,7 +171,7 @@ gdl_dock_master_class_init (GdlDockMasterClass *klass)
                              _("Default title for newly created floating docks"),
                              NULL,
                              G_PARAM_READWRITE));
-    
+
     g_object_class_install_property (
         object_class, PROP_LOCKED,
         g_param_spec_int ("locked", _("Locked"),
@@ -164,8 +189,14 @@ gdl_dock_master_class_init (GdlDockMasterClass *klass)
                            GDL_SWITCHER_STYLE_BOTH,
                            G_PARAM_READWRITE));
 
-    master_signals [LAYOUT_CHANGED] = 
-        g_signal_new ("layout-changed", 
+    /**
+     * GdlDockMaster::layout-changed:
+     *
+     * Signals that the layout has changed, one or more widgets have been moved,
+     * added or removed.
+     */
+    master_signals [LAYOUT_CHANGED] =
+        g_signal_new ("layout-changed",
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GdlDockMasterClass, layout_changed),
@@ -213,7 +244,7 @@ _gdl_dock_master_remove (GdlDockObject *object,
         if (object == master->controller) {
             GList *last;
             GdlDockObject *new_controller = NULL;
-            
+
             /* now find some other non-automatic toplevel to use as a
                new controller.  start from the last dock, since it's
                probably a non-floating and manual */
@@ -237,7 +268,7 @@ _gdl_dock_master_remove (GdlDockObject *object,
         }
     }
     /* disconnect dock object signals */
-    g_signal_handlers_disconnect_matched (object, G_SIGNAL_MATCH_DATA, 
+    g_signal_handlers_disconnect_matched (object, G_SIGNAL_MATCH_DATA,
                                           0, 0, NULL, NULL, master);
 
     /* unref the object from the hash if it's there */
@@ -325,13 +356,13 @@ gdl_dock_master_finalize (GObject *object)
     G_OBJECT_CLASS (gdl_dock_master_parent_class)->finalize (object);
 }
 
-static void 
+static void
 foreach_lock_unlock (GdlDockItem *item,
                      gboolean     locked)
 {
     if (!GDL_IS_DOCK_ITEM (item))
         return;
-    
+
     g_object_set (item, "locked", locked, NULL);
     if (gdl_dock_object_is_compound (GDL_DOCK_OBJECT (item)))
         gtk_container_foreach (GTK_CONTAINER (item),
@@ -344,7 +375,7 @@ gdl_dock_master_lock_unlock (GdlDockMaster *master,
                              gboolean       locked)
 {
     GList *l;
-    
+
     for (l = master->toplevel_docks; l; l = l->next) {
         GdlDock *dock = GDL_DOCK (l->data);
         if (dock->root)
@@ -413,7 +444,7 @@ gdl_dock_master_drag_begin (GdlDockItem *item,
 {
     GdlDockMaster  *master;
     GdlDockRequest *request;
-    
+
     g_return_if_fail (data != NULL);
     g_return_if_fail (item != NULL);
 
@@ -423,7 +454,7 @@ gdl_dock_master_drag_begin (GdlDockItem *item,
         master->priv->drag_request = g_new0 (GdlDockRequest, 1);
 
     request = master->priv->drag_request;
-    
+
     /* Set the target to itself so it won't go floating with just a click. */
     request->applicant = GDL_DOCK_OBJECT (item);
     request->target = GDL_DOCK_OBJECT (item);
@@ -431,46 +462,45 @@ gdl_dock_master_drag_begin (GdlDockItem *item,
     if (G_IS_VALUE (&request->extra))
         g_value_unset (&request->extra);
 
-    master->priv->rect_drawn = FALSE;
     master->priv->rect_owner = NULL;
 }
 
 static void
-gdl_dock_master_drag_end (GdlDockItem *item, 
+gdl_dock_master_drag_end (GdlDockItem *item,
                           gboolean     cancelled,
                           gpointer     data)
 {
     GdlDockMaster  *master;
     GdlDockRequest *request;
-    
+
     g_return_if_fail (data != NULL);
     g_return_if_fail (item != NULL);
 
     master = GDL_DOCK_MASTER (data);
     request = master->priv->drag_request;
-    
+
     g_return_if_fail (GDL_DOCK_OBJECT (item) == request->applicant);
-    
-    /* Erase previously drawn rectangle */
-    if (master->priv->rect_drawn)
-        gdl_dock_master_xor_rect (master);    
+
+    /* Hide the preview window */
+    gdl_dock_master_hide_preview (master);
 
     /* cancel conditions */
     if (cancelled || request->applicant == request->target)
         return;
-    
+
     /* dock object to the requested position */
     gdl_dock_object_dock (request->target,
                           request->applicant,
                           request->position,
                           &request->extra);
-    
+
     g_signal_emit (master, master_signals [LAYOUT_CHANGED], 0);
 }
 
 static void
-gdl_dock_master_drag_motion (GdlDockItem *item, 
-                             gint         root_x, 
+gdl_dock_master_drag_motion (GdlDockItem *item,
+                             GdkDevice *device,
+                             gint         root_x,
                              gint         root_y,
                              gpointer     data)
 {
@@ -482,30 +512,30 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
     gint            x, y;
     GdlDock        *dock = NULL;
     gboolean        may_dock = FALSE;
-    
+
     g_return_if_fail (item != NULL && data != NULL);
 
     master = GDL_DOCK_MASTER (data);
     request = master->priv->drag_request;
 
     g_return_if_fail (GDL_DOCK_OBJECT (item) == request->applicant);
-    
+
     my_request = *request;
 
     /* first look under the pointer */
-    window = gdk_window_at_pointer (&win_x, &win_y);
+    window = gdk_device_get_window_at_position (device, &win_x, &win_y);
     if (window) {
         GtkWidget *widget;
         /* ok, now get the widget who owns that window and see if we can
            get to a GdlDock by walking up the hierarchy */
         gdk_window_get_user_data (window, (gpointer) &widget);
         if (GTK_IS_WIDGET (widget)) {
-            while (widget && (!GDL_IS_DOCK (widget) || 
+            while (widget && (!GDL_IS_DOCK (widget) ||
 	           GDL_DOCK_OBJECT_GET_MASTER (widget) != master))
                 widget = gtk_widget_get_parent (widget);
             if (widget) {
                 gint win_w, win_h;
-                
+
                 widget_window = gtk_widget_get_window (widget);
 
                 /* verify that the pointer is still in that dock
@@ -551,7 +581,7 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
         }
     }
 
-  
+
     if (!may_dock) {
         GtkRequisition req;
 	/* Special case for GdlDockItems : they must respect the flags */
@@ -587,24 +617,20 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
 	&& dock != GDL_DOCK(master->controller))
 	    return;
 
-    if (!(my_request.rect.x == request->rect.x &&
-          my_request.rect.y == request->rect.y &&
-          my_request.rect.width == request->rect.width &&
-          my_request.rect.height == request->rect.height &&
-          dock == master->priv->rect_owner)) {
-
-        /* erase the previous rectangle */
-        if (master->priv->rect_drawn)
-            gdl_dock_master_xor_rect (master);
+    /* the previous windows is drawn by the dock master object if the preview
+     * is floating or by the corresponding dock object. It is not necessary
+     * the dock master could handle all cases but then it would be more
+     * difficult to implement different preview window in both cases */
+    /* erase previous preview window if necessary */
+    if (master->priv->rect_owner != dock)
+    {
+        gdl_dock_master_hide_preview (master);
+        master->priv->rect_owner = dock;
     }
 
-    /* set the new values */
+    /* draw the preview window */
     *request = my_request;
-    master->priv->rect_owner = dock;
-    
-    /* draw the previous rectangle */
-    if (~master->priv->rect_drawn)
-        gdl_dock_master_xor_rect (master);
+    gdl_dock_master_show_preview (master);
 }
 
 static void
@@ -621,40 +647,45 @@ _gdl_dock_master_foreach (gpointer key,
 }
 
 static void
-gdl_dock_master_xor_rect (GdlDockMaster *master)
+gdl_dock_master_show_preview (GdlDockMaster *master)
 {
-    cairo_rectangle_int_t *rect;
-    
     if (!master->priv || !master->priv->drag_request)
         return;
-    
-    master->priv->rect_drawn = ~master->priv->rect_drawn;
-    
-    if (master->priv->rect_owner) 
+
+    if (master->priv->rect_owner)
     {
-        if (master->priv->rect_drawn)
-        {
-            gdl_dock_xor_rect (master->priv->rect_owner,
+        gdl_dock_show_preview (master->priv->rect_owner,
                                &master->priv->drag_request->rect);
-        }
-        else
-        {
-            gdl_dock_xor_rect_hide(master->priv->rect_owner);
-            master->priv->rect_owner = NULL;
-        }
-    }        
-    if (master->priv->rect_drawn && !master->priv->rect_owner)
-    {
-            rect = &master->priv->drag_request->rect;
-
-            if (!master->priv->area_window)
-            {
-                master->priv->area_window = gdl_preview_window_new ();
-            }
-
-            gdl_preview_window_update (GDL_PREVIEW_WINDOW (master->priv->area_window), rect);
     }
-    else if (master->priv->area_window)
+    else
+    {
+        cairo_rectangle_int_t *rect;
+
+        rect = &master->priv->drag_request->rect;
+
+        if (!master->priv->area_window)
+        {
+            master->priv->area_window = gdl_preview_window_new ();
+        }
+
+        gdl_preview_window_update (GDL_PREVIEW_WINDOW (master->priv->area_window), rect);
+    }
+}
+
+static void
+gdl_dock_master_hide_preview (GdlDockMaster *master)
+{
+    cairo_rectangle_int_t *rect;
+
+    if (!master->priv)
+        return;
+
+    if (master->priv->rect_owner)
+    {
+        gdl_dock_hide_preview (master->priv->rect_owner);
+        master->priv->rect_owner = NULL;
+    }
+    if (master->priv->area_window)
     {
         gtk_widget_hide (master->priv->area_window);
     }
@@ -686,11 +717,11 @@ idle_emit_layout_changed (gpointer user_data)
 
     master->priv->idle_layout_changed_id = 0;
     g_signal_emit (master, master_signals [LAYOUT_CHANGED], 0);
-    
+
     return FALSE;
 }
 
-static void 
+static void
 item_dock_cb (GdlDockObject    *object,
               GdlDockObject    *requestor,
               GdlDockPlacement  position,
@@ -698,7 +729,7 @@ item_dock_cb (GdlDockObject    *object,
               gpointer          user_data)
 {
     GdlDockMaster *master = user_data;
-    
+
     g_return_if_fail (requestor && GDL_IS_DOCK_OBJECT (requestor));
     g_return_if_fail (master && GDL_IS_DOCK_MASTER (master));
 
@@ -713,13 +744,13 @@ item_dock_cb (GdlDockObject    *object,
     }
 }
 
-static void 
+static void
 item_detach_cb (GdlDockObject *object,
                 gboolean       recursive,
                 gpointer       user_data)
 {
     GdlDockMaster *master = user_data;
-    
+
     g_return_if_fail (object && GDL_IS_DOCK_OBJECT (object));
     g_return_if_fail (master && GDL_IS_DOCK_MASTER (master));
 
@@ -739,7 +770,7 @@ item_notify_cb (GdlDockObject *object,
     GdlDockMaster *master = user_data;
     gint locked = COMPUTE_LOCKED (master);
     gboolean item_locked;
-    
+
     g_object_get (object, "locked", &item_locked, NULL);
 
     if (item_locked) {
@@ -749,13 +780,20 @@ item_notify_cb (GdlDockObject *object,
         g_hash_table_remove (master->priv->locked_items, object);
         g_hash_table_insert (master->priv->unlocked_items, object, NULL);
     }
-    
+
     if (COMPUTE_LOCKED (master) != locked)
         g_object_notify (G_OBJECT (master), "locked");
 }
 
 /* ----- Public interface ----- */
 
+/**
+ * gdl_dock_master_add:
+ * @master: a #GdlDockMaster
+ * @object: a #GdlDockObject
+ *
+ * Add a new dock widget to the master.
+ */
 void
 gdl_dock_master_add (GdlDockMaster *master,
                      GdlDockObject *object)
@@ -764,13 +802,13 @@ gdl_dock_master_add (GdlDockMaster *master,
 
     if (!GDL_DOCK_OBJECT_AUTOMATIC (object)) {
         GdlDockObject *found_object;
-        
+
         /* create a name for the object if it doesn't have one */
         if (!object->name)
             /* directly set the name, since it's a construction only
                property */
             object->name = g_strdup_printf ("__dock_%u", master->priv->number++);
-        
+
         /* add the object to our hash list */
         if ((found_object = g_hash_table_lookup (master->dock_objects, object->name))) {
             g_warning (_("master %p: unable to add object %p[%s] to the hash.  "
@@ -782,15 +820,15 @@ gdl_dock_master_add (GdlDockMaster *master,
             g_hash_table_insert (master->dock_objects, g_strdup (object->name), object);
         }
     }
-    
+
     if (GDL_IS_DOCK (object)) {
         gboolean floating;
-        
+
         /* if this is the first toplevel we are adding, name it controller */
         if (!master->toplevel_docks)
             /* the dock should already have the ref */
             master->controller = object;
-        
+
         /* add dock to the toplevel list */
         g_object_get (object, "floating", &floating, NULL);
         if (floating)
@@ -824,7 +862,7 @@ gdl_dock_master_add (GdlDockMaster *master,
                               G_CALLBACK (item_notify_cb), master);
             item_notify_cb (object, NULL, master);
         }
-        
+
         /* If the item is notebook, set the switcher style */
         if (GDL_IS_DOCK_NOTEBOOK (object) &&
             GDL_IS_SWITCHER (GDL_DOCK_ITEM (object)->child))
@@ -832,7 +870,7 @@ gdl_dock_master_add (GdlDockMaster *master,
             g_object_set (GDL_DOCK_ITEM (object)->child, "switcher-style",
                           master->priv->switcher_style, NULL);
         }
-        
+
         /* post a layout_changed emission if the item is not automatic
          * (since it should be added to the items model) */
         if (!GDL_DOCK_OBJECT_AUTOMATIC (object)) {
@@ -843,6 +881,13 @@ gdl_dock_master_add (GdlDockMaster *master,
     }
 }
 
+/**
+ * gdl_dock_master_remove:
+ * @master: a #GdlDockMaster
+ * @object: a #GdlDockObject
+ *
+ * Remove one dock widget from the master.
+ */
 void
 gdl_dock_master_remove (GdlDockMaster *master,
                         GdlDockObject *object)
@@ -859,10 +904,10 @@ gdl_dock_master_remove (GdlDockMaster *master,
                 g_object_notify (G_OBJECT (master), "locked");
         }
     }
-        
+
     /* ref the master, since removing the controller could cause master disposal */
     g_object_ref (master);
-    
+
     /* all the interesting stuff happens in _gdl_dock_master_remove */
     _gdl_dock_master_remove (object, master);
 
@@ -873,11 +918,19 @@ gdl_dock_master_remove (GdlDockMaster *master,
             master->priv->idle_layout_changed_id =
                 g_idle_add (idle_emit_layout_changed, master);
     }
-    
+
     /* balance ref count */
     g_object_unref (master);
 }
 
+/**
+ * gdl_dock_master_foreach:
+ * @master: a #GdlDockMaster
+ * @function: (scope call): the function to call with each element's data
+ * @user_data: user data to pass to the function
+ *
+ * Call @function on each dock widget of the master.
+ */
 void
 gdl_dock_master_foreach (GdlDockMaster *master,
                          GFunc          function,
@@ -895,6 +948,16 @@ gdl_dock_master_foreach (GdlDockMaster *master,
     g_hash_table_foreach (master->dock_objects, _gdl_dock_master_foreach, &data);
 }
 
+/**
+ * gdl_dock_master_foreach_toplevel:
+ * @master: a #GdlDockMaster
+ * @include_controller: %TRUE to include the controller
+ * @function: (scope call): the function to call with each element's data
+ * @user_data: user data to pass to the function
+ *
+ * Call @function on each top level dock widget of the master, including or not
+ * the controller.
+ */
 void
 gdl_dock_master_foreach_toplevel (GdlDockMaster *master,
                                   gboolean       include_controller,
@@ -902,7 +965,7 @@ gdl_dock_master_foreach_toplevel (GdlDockMaster *master,
                                   gpointer       user_data)
 {
     GList *l;
-    
+
     g_return_if_fail (master != NULL && function != NULL);
 
     for (l = master->toplevel_docks; l; ) {
@@ -913,12 +976,21 @@ gdl_dock_master_foreach_toplevel (GdlDockMaster *master,
     }
 }
 
+/**
+ * gdl_dock_master_get_object:
+ * @master: a #GdlDockMaster
+ * @nick_name: the name of the dock widget.
+ *
+ * Looks for a #GdlDockObject named @nick_name.
+ *
+ * Returns: (allow-none) (transfer none): A #GdlDockObject named @nick_name or %NULL if it does not exist.
+ */
 GdlDockObject *
 gdl_dock_master_get_object (GdlDockMaster *master,
                             const gchar   *nick_name)
 {
     gpointer *found;
-    
+
     g_return_val_if_fail (master != NULL, NULL);
 
     if (!nick_name)
@@ -929,6 +1001,14 @@ gdl_dock_master_get_object (GdlDockMaster *master,
     return found ? GDL_DOCK_OBJECT (found) : NULL;
 }
 
+/**
+ * gdl_dock_master_get_controller:
+ * @master: a #GdlDockMaster
+ *
+ * Retrieves the #GdlDockObject acting as the controller.
+ *
+ * Returns: (transfer none): A #GdlDockObject.
+ */
 GdlDockObject *
 gdl_dock_master_get_controller (GdlDockMaster *master)
 {
@@ -937,6 +1017,13 @@ gdl_dock_master_get_controller (GdlDockMaster *master)
     return master->controller;
 }
 
+/**
+ * gdl_dock_master_set_controller:
+ * @master: a #GdlDockMaster
+ * @new_controller: a #GdlDockObject
+ *
+ * Set a new controller. The controller must be a top level #GdlDockObject.
+ */
 void
 gdl_dock_master_set_controller (GdlDockMaster *master,
                                 GdlDockObject *new_controller)
@@ -947,7 +1034,7 @@ gdl_dock_master_set_controller (GdlDockMaster *master,
         if (GDL_DOCK_OBJECT_AUTOMATIC (new_controller))
             g_warning (_("The new dock controller %p is automatic.  Only manual "
                          "dock objects should be named controller."), new_controller);
-        
+
         /* check that the controller is in the toplevel list */
         if (!g_list_find (master->toplevel_docks, new_controller))
             gdl_dock_master_add (master, new_controller);
@@ -964,19 +1051,19 @@ static void
 set_switcher_style_foreach (GtkWidget *obj, gpointer user_data)
 {
     GdlSwitcherStyle style = GPOINTER_TO_INT (user_data);
-    
+
     if (!GDL_IS_DOCK_ITEM (obj))
         return;
-    
+
     if (GDL_IS_DOCK_NOTEBOOK (obj)) {
-        
+
         GtkWidget *child = GDL_DOCK_ITEM (obj)->child;
         if (GDL_IS_SWITCHER (child)) {
-            
+
             g_object_set (child, "switcher-style", style, NULL);
         }
     } else if (gdl_dock_object_is_compound (GDL_DOCK_OBJECT (obj))) {
-        
+
         gtk_container_foreach (GTK_CONTAINER (obj),
                                set_switcher_style_foreach,
                                user_data);
@@ -989,7 +1076,7 @@ gdl_dock_master_set_switcher_style (GdlDockMaster *master,
 {
     GList *l;
     g_return_if_fail (GDL_IS_DOCK_MASTER (master));
-    
+
     master->priv->switcher_style = switcher_style;
     for (l = master->toplevel_docks; l; l = l->next) {
         GdlDock *dock = GDL_DOCK (l->data);
